@@ -1,15 +1,14 @@
-# Copyright (c) 2016, 2018-2020 Claudiu Popa <pcmanticore@gmail.com>
-# Copyright (c) 2017 Łukasz Rogalski <rogalski.91@gmail.com>
-# Copyright (c) 2020-2021 hippo91 <guillaume.peillex@gmail.com>
-# Copyright (c) 2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
-
 # Licensed under the LGPL: https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html
-# For details: https://github.com/PyCQA/astroid/blob/master/COPYING.LESSER
-import astroid
+# For details: https://github.com/PyCQA/astroid/blob/main/LICENSE
+# Copyright (c) https://github.com/PyCQA/astroid/blob/main/CONTRIBUTORS.txt
+
+from astroid.brain.helpers import register_module_extender
+from astroid.builder import parse
+from astroid.manager import AstroidManager
 
 
 def _thread_transform():
-    return astroid.parse(
+    return parse(
         """
     class lock(object):
         def acquire(self, blocking=True, timeout=-1):
@@ -29,4 +28,4 @@ def _thread_transform():
     )
 
 
-astroid.register_module_extender(astroid.MANAGER, "threading", _thread_transform)
+register_module_extender(AstroidManager(), "threading", _thread_transform)
