@@ -1,17 +1,6 @@
-# Copyright (c) 2006-2011, 2013-2014 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
-# Copyright (c) 2013 Phil Schaf <flying-sheep@web.de>
-# Copyright (c) 2014-2020 Claudiu Popa <pcmanticore@gmail.com>
-# Copyright (c) 2014-2015 Google, Inc.
-# Copyright (c) 2014 Alexander Presnyakov <flagist0@gmail.com>
-# Copyright (c) 2015-2016 Ceridwen <ceridwenv@gmail.com>
-# Copyright (c) 2016 Derek Gustafson <degustaf@gmail.com>
-# Copyright (c) 2017 Łukasz Rogalski <rogalski.91@gmail.com>
-# Copyright (c) 2018 Anthony Sottile <asottile@umich.edu>
-# Copyright (c) 2020-2021 hippo91 <guillaume.peillex@gmail.com>
-# Copyright (c) 2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
-
 # Licensed under the LGPL: https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html
-# For details: https://github.com/PyCQA/astroid/blob/master/COPYING.LESSER
+# For details: https://github.com/PyCQA/astroid/blob/main/LICENSE
+# Copyright (c) https://github.com/PyCQA/astroid/blob/main/CONTRIBUTORS.txt
 
 """The AstroidBuilder makes astroid from living object and / or from _ast
 
@@ -19,19 +8,18 @@ The builder is not thread safe and can't be used to parse different sources
 at the same time.
 """
 
+from __future__ import annotations
+
 import os
 import textwrap
+import types
 from tokenize import detect_encoding
 
+from astroid import bases, modutils, nodes, raw_building, rebuilder, util
 from astroid._ast import get_parser_module
-from astroid import bases
-from astroid import exceptions
-from astroid import manager
-from astroid import modutils
-from astroid import raw_building
-from astroid import rebuilder
-from astroid import nodes
-from astroid import util
+from astroid.exceptions import AstroidBuildingError, AstroidSyntaxError, InferenceError
+from astroid.manager import AstroidManager
+from astroid.nodes.node_classes import NodeNG
 
 objects = util.lazy_import("objects")
 
@@ -44,10 +32,10 @@ _TRANSIENT_FUNCTION = "__"
 # when calling extract_node.
 _STATEMENT_SELECTOR = "#@"
 MISPLACED_TYPE_ANNOTATION_ERROR = "misplaced type annotation"
-MANAGER = manager.AstroidManager()
 
 
 def open_source_file(filename):
+    # pylint: disable=consider-using-with
     with open(filename, "rb") as byte_stream:
         encoding = detect_encoding(byte_stream.readline)[0]
     stream = open(filename, newline=None, encoding=encoding)
@@ -63,7 +51,7 @@ def _can_assign_attr(node, attrname):
     else:
         if slots and attrname not in {slot.value for slot in slots}:
             return False
-    return True
+    return node.qname() != "builtins.object"
 
 
 class AstroidBuilder(raw_building.InspectBuilder):
@@ -76,19 +64,27 @@ class AstroidBuilder(raw_building.InspectBuilder):
     by default being True.
     """
 
-    # pylint: disable=redefined-outer-name
     def __init__(self, manager=None, apply_transforms=True):
-        super().__init__()
-        self._manager = manager or MANAGER
+        super().__init__(manager)
         self._apply_transforms = apply_transforms
 
-    def module_build(self, module, modname=None):
+    def module_build(
+        self, module: types.ModuleType, modname: str | None = None
+    ) -> nodes.Module:
         """Build an astroid from a living module instance."""
         node = None
         path = getattr(module, "__file__", None)
-        if path is not None:
+        loader = getattr(module, "__loader__", None)
+        # Prefer the loader to get the source rather than assuming we have a
+        # filesystem to read the source file from ourselves.
+        if loader:
+            modname = modname or module.__name__
+            source = loader.get_source(modname)
+            if source:
+                node = self.string_build(source, modname, path=path)
+        if node is None and path is not None:
             path_, ext = os.path.splitext(modutils._path_from_filename(path))
-            if ext in (".py", ".pyc", ".pyo") and os.path.exists(path_ + ".py"):
+            if ext in {".py", ".pyc", ".pyo"} and os.path.exists(path_ + ".py"):
                 node = self.file_build(path_ + ".py", modname)
         if node is None:
             # this is a built-in module
@@ -108,14 +104,14 @@ class AstroidBuilder(raw_building.InspectBuilder):
         try:
             stream, encoding, data = open_source_file(path)
         except OSError as exc:
-            raise exceptions.AstroidBuildingError(
+            raise AstroidBuildingError(
                 "Unable to load file {path}:\n{error}",
                 modname=modname,
                 path=path,
                 error=exc,
             ) from exc
         except (SyntaxError, LookupError) as exc:
-            raise exceptions.AstroidSyntaxError(
+            raise AstroidSyntaxError(
                 "Python 3 encoding specification error or unknown encoding:\n"
                 "{error}",
                 modname=modname,
@@ -124,7 +120,7 @@ class AstroidBuilder(raw_building.InspectBuilder):
             ) from exc
         except UnicodeError as exc:  # wrong encoding
             # detect_encoding returns utf-8 if no encoding specified
-            raise exceptions.AstroidBuildingError(
+            raise AstroidBuildingError(
                 "Wrong or no encoding specified for {filename}.", filename=path
             ) from exc
         with stream:
@@ -135,27 +131,29 @@ class AstroidBuilder(raw_building.InspectBuilder):
                 except ImportError:
                     modname = os.path.splitext(os.path.basename(path))[0]
             # build astroid representation
-            module = self._data_build(data, modname, path)
-            return self._post_build(module, encoding)
+            module, builder = self._data_build(data, modname, path)
+            return self._post_build(module, builder, encoding)
 
     def string_build(self, data, modname="", path=None):
         """Build astroid from source code string."""
-        module = self._data_build(data, modname, path)
+        module, builder = self._data_build(data, modname, path)
         module.file_bytes = data.encode("utf-8")
-        return self._post_build(module, "utf-8")
+        return self._post_build(module, builder, "utf-8")
 
-    def _post_build(self, module, encoding):
+    def _post_build(
+        self, module: nodes.Module, builder: rebuilder.TreeRebuilder, encoding: str
+    ) -> nodes.Module:
         """Handles encoding and delayed nodes after a module has been built"""
         module.file_encoding = encoding
         self._manager.cache_module(module)
         # post tree building steps after we stored the module in the cache:
-        for from_node in module._import_from_nodes:
+        for from_node in builder._import_from_nodes:
             if from_node.modname == "__future__":
                 for symbol, _ in from_node.names:
                     module.future_imports.add(symbol)
             self.add_from_names_to_locals(from_node)
         # handle delayed assattr nodes
-        for delayed in module._delayed_assattr:
+        for delayed in builder._delayed_assattr:
             self.delayed_assattr(delayed)
 
         # Visit the transforms
@@ -163,12 +161,14 @@ class AstroidBuilder(raw_building.InspectBuilder):
             module = self._manager.visit_transforms(module)
         return module
 
-    def _data_build(self, data, modname, path):
+    def _data_build(
+        self, data: str, modname, path
+    ) -> tuple[nodes.Module, rebuilder.TreeRebuilder]:
         """Build tree node from data and add some informations"""
         try:
             node, parser_module = _parse_string(data, type_comments=True)
         except (TypeError, ValueError, SyntaxError) as exc:
-            raise exceptions.AstroidSyntaxError(
+            raise AstroidSyntaxError(
                 "Parsing Python code failed:\n{error}",
                 source=data,
                 modname=modname,
@@ -188,18 +188,18 @@ class AstroidBuilder(raw_building.InspectBuilder):
                 path is not None
                 and os.path.splitext(os.path.basename(path))[0] == "__init__"
             )
-        builder = rebuilder.TreeRebuilder(self._manager, parser_module)
+        builder = rebuilder.TreeRebuilder(self._manager, parser_module, data)
         module = builder.visit_module(node, modname, node_file, package)
-        module._import_from_nodes = builder._import_from_nodes
-        module._delayed_assattr = builder._delayed_assattr
-        return module
+        return module, builder
 
     def add_from_names_to_locals(self, node):
         """Store imported names to the locals
 
         Resort the locals if coming from a delayed node
         """
-        _key_func = lambda node: node.fromlineno
+
+        def _key_func(node):
+            return node.fromlineno
 
         def sort_locals(my_list):
             my_list.sort(key=_key_func)
@@ -208,7 +208,7 @@ class AstroidBuilder(raw_building.InspectBuilder):
             if name == "*":
                 try:
                     imported = node.do_import_module()
-                except exceptions.AstroidBuildingError:
+                except AstroidBuildingError:
                     continue
                 for name in imported.public_names():
                     node.parent.set_local(name, node)
@@ -223,7 +223,7 @@ class AstroidBuilder(raw_building.InspectBuilder):
         This adds name to locals and handle members definition.
         """
         try:
-            frame = node.frame()
+            frame = node.frame(future=True)
             for inferred in node.expr.infer():
                 if inferred is util.Uninferable:
                     continue
@@ -252,17 +252,17 @@ class AstroidBuilder(raw_building.InspectBuilder):
                 if (
                     frame.name == "__init__"
                     and values
-                    and values[0].frame().name != "__init__"
+                    and values[0].frame(future=True).name != "__init__"
                 ):
                     values.insert(0, node)
                 else:
                     values.append(node)
-        except exceptions.InferenceError:
+        except InferenceError:
             pass
 
 
-def build_namespace_package_module(name, path):
-    return nodes.Module(name, doc="", path=path, package=True)
+def build_namespace_package_module(name: str, path: list[str]) -> nodes.Module:
+    return nodes.Module(name, path=path, package=True)
 
 
 def parse(code, module_name="", path=None, apply_transforms=True):
@@ -276,7 +276,9 @@ def parse(code, module_name="", path=None, apply_transforms=True):
         don't want the default transforms to be applied.
     """
     code = textwrap.dedent(code)
-    builder = AstroidBuilder(manager=MANAGER, apply_transforms=apply_transforms)
+    builder = AstroidBuilder(
+        manager=AstroidManager(), apply_transforms=apply_transforms
+    )
     return builder.string_build(code, modname=module_name, path=path)
 
 
@@ -334,7 +336,7 @@ def _find_statement_by_line(node, line):
       can be found.
     :rtype:  astroid.bases.NodeNG or None
     """
-    if isinstance(node, (nodes.ClassDef, nodes.FunctionDef)):
+    if isinstance(node, (nodes.ClassDef, nodes.FunctionDef, nodes.MatchCase)):
         # This is an inaccuracy in the AST: the nodes that can be
         # decorated do not carry explicit information on which line
         # the actual definition (class/def), but .fromline seems to
@@ -354,7 +356,7 @@ def _find_statement_by_line(node, line):
     return None
 
 
-def extract_node(code, module_name=""):
+def extract_node(code: str, module_name: str = "") -> NodeNG | list[NodeNG]:
     """Parses some Python code as a module and extracts a designated AST node.
 
     Statements:
@@ -406,7 +408,6 @@ def extract_node(code, module_name=""):
     a module. Will be passed through textwrap.dedent first.
     :param str module_name: The name of the module.
     :returns: The designated node from the parse tree, or a list of nodes.
-    :rtype: astroid.bases.NodeNG, or a list of nodes.
     """
 
     def _extract(node):
@@ -440,6 +441,14 @@ def extract_node(code, module_name=""):
     return extracted
 
 
+def _extract_single_node(code: str, module_name: str = "") -> NodeNG:
+    """Call extract_node while making sure that only one value is returned."""
+    ret = extract_node(code, module_name)
+    if isinstance(ret, list):
+        return ret[0]
+    return ret
+
+
 def _parse_string(data, type_comments=True):
     parser_module = get_parser_module(type_comments=type_comments)
     try:
